// GEN = GenRegister()
// DEF = power-on-default
// WDG = Watchdog Timer
// MAP = digital Matrix Mapping
// CFW = Cypress Framework (common to all ACCES USB devices)
// PWM = Pulse Width Modified digital output

#ifndef __ACCES_FX2_USB_VR_H__
#define __ACCES_FX2_USB_VR_H__

#include "card_specific.h"

#define DEFREV_MINOR (0x09)
#define DEFREV_MAJOR (0x00)
#define PWM_MAX_TABLELENGTH (8)
#define MAP_MAX_TABLELENGTH (32)

#define DRL_EEPROM_START (0x1D00)

/*
Bytes at SETUPDAT:
   [0]USB request type, direction, and recipient; not sure of values, but will be "vendor request" type and in or out
   [1]Vendor request code
   [2]Value(LSB), defined by us per transfer
   [3]Value(MSB)
   [4]Index(LSB), defined by us per transfer
   [5]Index(MSB)
   [6]Length(LSB), number of bytes transferred in data stage
   [7]Length(MSB)
*/

#define ValueLSB  (SETUPDAT[2])
#define ValueMSB  (SETUPDAT[3])
#define IndexLSB  (SETUPDAT[4])
#define IndexMSB  (SETUPDAT[5])
#define LengthLSB (SETUPDAT[6])
#define LengthMSB (SETUPDAT[7])

#define USB_WRITE                (( SETUPDAT[0] & 0x80 ) == 0x00 )
#define USB_READ                 (( SETUPDAT[0] & 0x80 ) == 0x80 )


#define YES TRUE
#define NO FALSE

#define WordFromBytes(LSB, MSB) ((WORD)(LSB) | ((MSB) << 8))
// intel byte order
#define WordFromEP0(EPLSB) ((WORD)WordFromBytes(EP0BUF[(EPLSB)], EP0BUF[(EPLSB+1)]))
#define RD_CLK_ADR         0x17  // OSCILLATOR READ CLOCK on I2C bus
//*************************************************************************************************
// USB VR #defines
//*************************************************************************************************
#define AUR_DIO_WRITE            0x10  // write all 32 bits of output
#define AUR_DIO_READ             0x11  // read all 32 bits of input
#define AUR_DIO_CONFIG           0x12  // configure which bits are I/O, COS mask, and tristate bit
#define AUR_DIO_WRITE_ANY        0x1F  // configure and write with masks - read/modify/write both config and digital data
#define AUR_DIO_READ_CONFIG      0x13
#define AUR_DIO_READ_LATCHED     0x14
#define AUR_DIO_READ_DEBOUNCED   0x15
#define AUR_DIO_ADVANCED         0x18  // Digital I/O with features (pulse generation, freq out, PWM, debounce, etc)

#define AUR_CTR_READ             0x20  // CtrRead
#define AUR_CTR_MODE             0x21  // CtrMode //useful to turn off counter, or use as extra DO
#define AUR_CTR_LOAD             0x22
#define AUR_MODELOAD             0x23  // CtrModeLoad
#define AUR_CTR_FREQGATE         0x24  // select a counter as the GATE source for FREQ measurement (CTR = 0..14)
#define AUR_CTR_READ_ALL         0x25  // CtrRead x 15 (reads ALL counters)
#define AUR_CTR_READ_LATCH_ALL   0x26  // read the global variables storing last latched counters from all 15 counters
/*
  The AUR_CTR_READ_LATCH_ALL VR command is designed to allow as many as 14 counters to be used to measure frequency simultaneously and easily.
  The customer must issue AUR_MODELOAD commands to all 14 frequency counters for MODE "2", LOAD should be "0" but that's up to their program.
  The customer must issue AUR_MODELOAD for the SELECTED GATE source (selected using AUR_CTR_FREQGATE) to produce a slow enough MODE 3 output.
  The customer *THEN* issues AUR_CTR_FREQGATE to indicate "begin measuring frequencies using FREQGATE as the GATE source"
  The 8051 is now storing counts onboard, measurement has STARTed
  The customer can poll all the counters using AUR_CTR_READ_LATCH_ALL any time after START.  The data returned is the count value most recently
    latched by high-side gate periods.  If the customer reads less data than the FX2 acquires, the 31st byte will be how many potential reads were
   missed.  If the customer reads more often than new data is available, the 32st byte will be 0 until new data is ready.  Ideally the 31st data
   byte would always read "1", indicating the FX2 took data one time for each read of the data by the customer's program.
*/
#define AUR_CTR_MODELOAD_ALL     0x27

#define AUR_DEFAULTS_CONFIG      0x28  // Generic Power-On Defaults Interface 2.0 (DEF)
#define AUR_WATCHDOG_CONFIG      0x2E  // Generic Watchdog Interface (WDG) 2.0

#define VR_WATCHDOG_CONFIG       0x2F  // deprecated Watchdog Interface

#define AUR_GLO_QUERY            0x30  // read current config
#define AUR_GLO_SAVE             0x31  // write all config into EEPROM

#define VR_FLUSH_FIFO_NOW        0x32

#define VR_CLEAR_FIFO_NEXT       0x34
#define VR_CLEAR_FIFO_NOW        0x35
#define VR_CLEAR_FIFO_HANG       0x36
#define VR_CLEAR_FIFO_WAIT       0x36
#define AUR_GEN_ABORT_AND_CLEAR  0x38
#define GPIF_ABORT_AND_CLEAR     0x38
#define SET_GPIF_MODE            0x39  // pass "1" for continuous, "0" for TC (USB-AIx only for now)

#define AUR_GEN_REGISTER         0x3C
// GenRegister creates a bank of registers to control various aspects of devices.
//     Index=="1" is "bit ADC_Continuous mode", with ValueLSb as the setting
//     Index=="2" is "Write Current Init Configuration to EEPROM" with Value == "0xACCE" being a confirmation flag, nothing happens otherwise
#define AUR_QuerySimplePNP       0x3F

#define AUR_DACDIO_WRITEALL      0x8C

// B0 through BF are set aside for self tests.
#define AUR_SELF_TEST_1          0x91  // perform Self Test #1 implemented in USB-AI16-16 family

// Cypress reserves A0 through AF

#define AUR_DAC_DIRECT           0xB3  // STANDARD SINGLE OR MULTI DAC API
#define AUR_QUERY_EMPTY          0xB4  // ?? not working, move off this VR, using in USB-DIO-16H but something needed for all FIFO modes

#define AUR_CAL                  0xB5  // upload a usb-ao CAL file for one channel (64K words) or flexible --J2H

#define AUR_FACTORY_CAL          0xB6  // Choose factory or customer usb-ao calibration tables
#define AUR_GET_AD_SCAN          0xB6  // read an ADC scan immediately

#define AUR_DAC_RANGE            0xB7  // select DAC range and calibration table; enable voltage references
#define AUR_DAC_SLOW_WAVEFORM    0xB8  // upload waveform for slow playback by MCU
#define AUR_PROBE_CAL            0xBA  // Probe Cal Feature      Must get "0xBB" back in order to use vendor request 0xBB to upload Calibration

#define AUR_WRITE_CAL            0xBB  // GPIF write CAL
#define VX_BB                    0xBB  // GPIF write DIO

#define AUR_READ_DATA            0xBC  // GPIF read Data
#define VX_BC                    0xBC  // GPIF read DIO

#define VR_SETCLOCKS             0xBD  // USB-DIO-16H dual frequency sources, internal/external select
#define AUR_AI16_INIT            0xBE
#define AUR_ADCSETGETConfig      0xBE
#define AUR_ADC_IMMEDIATE        0xBF  // ADC_GETIMMEDIATE


// The 0xC0-0xF0 range is reserved for "Custom" Vendor Specific Requests - overlap/re-use of numbers is allowed, as long as different hardware
#define AUR_SPI_WRITE            0xC0
#define AUR_SPI_READ             0xC1
#define AUR_ADVANCED_RECONNAISSANCE_CORP_USB_AIx 0xC2
#define AUR_CSA_JEFF_SMITH       0xC3
#define AUR_CAMSYNC              0xC4
#define AUR_CSA_REPLAY_TECHNOLOGIES 0xC5
#define AUR_CSA_DAC_SlowRamp     0xC6 // -- added 20140519, primarily for Artur Olszak, aolszak@apre-inst.com
// The 0xC0-0xEF range is reserved for "Custom" Vendor Specific Requests.

#define AUR_AI16_INIT_READ       0xD2
#define AUR_ADCGETSETConfig      0xD2

#define AUR_DIOQuadratureRead    0xE1 // quadrature stuff.

#define AUR_FLASH_1TO1           0xF0  // set the Cal Flash to default 1-to-1 tables
#define AUR_ERASE_FLASH          0xFC  // erase a sector or the whole chip (index==0xff)


// * * * * *   C O M M O N   F U N C T I O N   D E F I N I T I O N S   * * * * * * * * * * * * * * * * * *

BOOL HandleFW20VRs();

// * * * * *   C Y P R E S S   D E F I N I T I O N S   * * * * * * * * * * * * * * * * * * * * * * * * * *

// the following are defined by Cypress as part of the vendAX.c sample file, but we use them for our purposes sometimes.
// Confirm upload/download are needed --J2H delete if not

#define EP0BUFF_SIZE             0x40
#define VR_UPLOAD                0xc0
#define VR_DOWNLOAD               0x40
#define VR_EEPROM                0xa2 // loads (uploads) EEPROM
#define VR_RAM                   0xa3 // loads (uploads) external ram
#define GPIFTRIGRD 4
#define GPIF_EP2 0
#define GPIF_EP4 1
#define GPIF_EP6 2
#define GPIF_EP8 3
#define SERIAL_ADDR              0x50

// Endpoint configuration definitions
#define BYTE_WIDE               0x00        // configure as byte-wide data bus
#define WORD_WIDE               0x01        // configure as word-wide data bus
#define ZERO_LEN_IN             0x04        // insert definition here
#define AUTO_IN                   0x08      // commit packets automatically (in)
#define AUTO_OUT                0x10        // commit packets automatically (out)
#define OEP1                    0x20        // insert definition here
#define INFM1                   0x40        // insert definition here
#define doVbus()  if (!Vbus) USBCS |= bmDISCON; else USBCS &= ~bmDISCON

sbit IOC0 = 0xA0+0;
sbit IOC1 = 0xA0+1;
sbit IOC2 = 0xA0+2;
sbit IOC3 = 0xA0+3;
sbit IOC4 = 0xA0+4;
sbit IOC5 = 0xA0+5;
sbit IOC6 = 0xA0+6;
sbit IOC7 = 0xA0+7;

sbit IOD0 = 0xB0+0;
sbit IOD1 = 0xB0+1;
sbit IOD2 = 0xB0+2;
sbit IOD3 = 0xB0+3;
sbit IOD4 = 0xB0+4;
sbit IOD5 = 0xB0+5;
sbit IOD6 = 0xB0+6;
sbit IOD7 = 0xB0+7;

BOOL TD_Suspend(void);        // Called before the device goes into suspend mode
BOOL TD_Resume(void);         // Called after the device resumes
BOOL DR_GetDescriptor(void);
BOOL DR_SetConfiguration(void);   // Called when a Set Configuration command is received
BOOL DR_GetConfiguration(void);   // Called when a Get Configuration command is received
BOOL DR_SetInterface(void);       // Called when a Set Interface command is received
BOOL DR_GetInterface(void);       // Called when a Set Interface command is received
BOOL DR_GetStatus(void);
BOOL DR_ClearFeature(void);
BOOL DR_SetFeature(void);
void VRRAMEEPROM();
void WDG_VR();

// * * * * *   C F W   D E F I N I T I O N S   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
extern BOOL GotSUD;        // Received setup data flag
extern BOOL Sleep;
extern BOOL Rwuen;
extern BOOL Selfpwr;

// void  ResetEP6FIFO(void);
// void  ResetEP2FIFO(void);
void ResetEP2andEP6FIFOs(void);
void WhileWaitingForDataPacket( void );

void OverwriteStringDscrM(char *newstr);
void EEPROMWriteByte(WORD addr, BYTE value);
void EEPROMWrite(WORD addr, BYTE length, BYTE xdata *buf); //TPM EEPROM Write
void EEPROMRead(WORD addr, BYTE length, BYTE xdata *buf);  //TPM EEPROM Read

// * * * * *   G E N R E G I S T E R   D E F I N I T I O N S   * * * * * * * * * * * * * * * * * * * * * *
void GEN_Register_VR(void);

// * * * * *   W A T C H D O G   D E F I N I T I O N S   * * * * * * * * * * * * * * * * * * * * * * * * *
#define _ACTIVITY_LED_ON_TIME_  (0xFF)
#define _ACTIVITY_LED_OFF_TIME_ (0x500)
#define _ACTIVITY_LED_TIME_     (_ACTIVITY_LED_ON_TIME_ + _ACTIVITY_LED_OFF_TIME_)

#define EEPROM_WDG_START (0x1D80)
#define petWatchdog()  (Seconds = 0, Tenths = 0)

void WDG_Go();
#if CTR_NumBlocks > 0
void WDG_8254Init();
#endif

extern bit fBark;
extern bit fActivity;

extern BYTE Configuration;    // Current configuration
extern BYTE AlternateSetting; // Alternate settings

void WDG_Init(void);
void Activity(void);
void WDG_WriteToEEPROM(void);
void WDG_Bark(void);
void WDG_ReadFromEEPROM(void);


extern BYTE xdata T2DivisorH;
extern BYTE xdata T2DivisorL;
#endif